;
;**************************************************************************************************
;DOS/65 C02 (CMOS) ROM Version
;
;This is Version 3.04 (KM 2022)
;This is the core of the ROM resident version of DOS/65 (CMOS version)
;and is derived from SYSGEN V3.02 by Richard A. Leary
;
; The target platform for this build is:
;  A prototype 3.3-Volt system based on the C02 Pocket SBC - KM
;  This prototype uses a NXP 28L92 DUART and a Hitachi MicroDrive PATA interface
;  The rest of the system is mainly the same, i.e., 32KB RAM, 32KB ROM, 8MHz CPU
;**************************************************************************************************
; Assembler directives
;
                PW      132
                PL      66
                GLOBALS ON
                CASE    OFF
                INCLIST ON
;**************************************************************************************************
;
; The following is history from Richard's initial ROM version and includes my history to get
; to this particular version. I've also broken out the main code modules and use "include" to
; add them into the main code. The Code Modules are:
;       DOS65-MAIN-V304.ASM (This file)
;       DOS65-CCM-V304.ASM - Concurrent Command Module
;       DOS65-PEM-V304.ASM - Primitive Execution Module
;       DOS65-SIM-V304.ASM - System Interface Module
;
; Other files required contain system variables and constants for C02BIOS and C02Monitor
;  The current BIOS and Monitor versions for the Prototype system are 4.01
;
;**************************************************************************************************
;
;Changes from V2.1 to V3.0
;include:
;               added USER command and use
;               added ability to input .KIM & .HEX
;               added batch file functions
;               made directory entries per line a parameter
;               implemented use of IOSTAT to save & pass
;               drive number and user code
;               made use of SWITCH always the case
;
;released:      24 February 2015 (3.00S)
;last revised:
;               2 March 2015 (3.01S)
;                       added check of number actual sectors in image write
;                       made trkcnt separate variable
;               21 August 2015 (3.02)
;                       corrected file name parsing error
;                       set dir entries per line to 2
;               2 July 2017 (3.02W ROM)
;                       conversion to ROM
;                       eliminated pem page 0 swap
;                       corrected sysdef location
;                       changed sim references to simram
;                       set 4 wide directory in ccm
;
;**************************************************************************************************
;
;               14 March 2021 - KM (1st adaptation)
;Several changes to structure - saves RAM usage
; Page Zero start at $80 (Monitor/BIOS uses $A0 - $FF)
; Condensed defined area - removed all SD related code
; Removed DEFS for allocation maps, RAM initialized at coldstart
; Moved SIM to a half-page boundary (frees up another chunk of RAM)
; Added allocation maps for three additional drives
; Coldstart requires 2.0 pages of RAM initialized
; Warmstart requires 1.5 pages of RAM initialized
; Fixed a problem with invalid drive looping on error output
; Reduced ROM image size ~3KB smaller
;
;               11 April 2021 - KM (Version 3.03)
;This new version contains an updated CCM section using CMOS
; instructions and addressing modes, is smaller in size (2046 bytes)
; Several performance improvements due to CMOS updates
;
;This new version contains an updated PEM section using CMOS
; instructions and addressing modes, is smaller in size (2776 bytes)
; Several performance improvements due to CMOS updates
;
;This program contains the complete DOS/65 operating system, including:
; CCM, PEM and SIM. All are assembled and linked to a single Intel HEX file.
; This can be loaded into ROM, but still require that the C02BIOS and C02Monitor
; are also loaded into the ROM as well. Note that the SIM section calls several
; functions from the BIOS/Monitor code via the integrated JMP tables.
; DOS/65 is booted from ROM via the C02Monitor using "Ctrl-B".
;
; ROM size is 6.5KB, which includes 512 bytes allocated for CCM/PEM/SIM RAM.
; This is copied from ROM to RAM during a Cold start for memory usage which is
; dynamic when DOS/65 is running.
;
;               11 October 2021 - KM (3.03)
;Minor update for hardware changes:
; SC28L92 DUART replaces SCC2691 - requires relocating some Page Zero variables
;  no functional change... albeit a new INCLUDE file for BIOS/Monitor JMP tables
;
;               2 March 2022 - KM (3.04)
;Update to Version 3.04:
; Updated hardware platform BIOS and Monitor code now at 4.01
; Adding support for second serial port via Reader/Punch
; Some additional code streamlining to reduce size and improve performance
; - CCM size now at 2036 bytes
; - PEM size now at 2751 bytes
; - SIM size now at 1091 bytes
;
;**************************************************************************************************
;
;Base addresses and definitions
pzstrt          equ     $80             ;start of page zero system RAM
btejmp          equ     $100            ;warm boot jump
pemjmp          equ     $103            ;jump to PEM
iostat          equ     $106            ;I/O status
dflfcb          equ     $107            ;default FCB
dflbuf          equ     $128            ;default buffer
tea             equ     $800            ;TEA start - KM
;C02 Pocket SBC memory parameters
ramstrt         equ     $7000           ;start of high ram used by system - KM
romstrt         equ     $C000           ;default ROM location - KM
romorg          equ     $D800           ;changed for condensed RAM image - KM
hstbuf          equ     $0600           ;moved host buffer to low RAM - KM
;
;Fixed parameters
lf              equ     $0A             ;linefeeed
cr              equ     $0D             ;return
eof             equ     $1A             ;end of file
null            equ     0               ;null
ctlc            equ     3               ;abort
ctle            equ     5               ;physical cr lf
ctli            equ     9               ;tab character
ctlp            equ     $10             ;toggle printer
ctlr            equ     $12             ;repeat line
ctls            equ     $13             ;freeze
ctlx            equ     $18             ;cancel
semico          equ     $3B             ;semicolon
delete          equ     $7F             ;delete character
numcmd          equ     37              ;number of PEM commands

;Page Zero use
                page0
;CCM / PEM
addinp          equ     pzstrt          ;initialized to a,y
bufadd          equ     addinp+2        ;buffer address
alcpnt          equ     bufadd+2        ;allocation map pointer
chkpnt          equ     alcpnt+2        ;checksum map pointer
msgptr          equ     chkpnt+2        ;message pointer
movptr          equ     msgptr          ;and move pointer
dcbloc          equ     msgptr+2        ;pointer to dcb
;
;sim
sim_dmaadr      equ     dcbloc+2        ;record buffer
sim_dcbpnt      equ     sim_dmaadr+2    ;pointer into selected dcb
sim_movepnt     equ     sim_dcbpnt+2    ;read or write move host pointer
;
;mon/bios
IDE_LBA0        equ     sim_movepnt+2   ;IDE LBA Variables for DOS/65
IDE_LBA1        equ     sim_movepnt+3
IDE_LBA2        equ     sim_movepnt+4
;
mon_nxtdrv      equ     IDE_LBA2+1      ;SIM access variables
mon_nxttrk      equ     IDE_LBA2+2
mon_nxtsec      equ     IDE_LBA2+3
;
                ends
;
dosram          section indirect ramstrt
;
;RAM code and variables
;
;CCM section
;
ccmram          jmp     ccm
;input buffer
cnsbuf
cnsmax          byte    127
cnslng          byte    0
cnstxt          defs    128             ;set to 128 for safety
;fcb one
fcbone          defs    33
;
;SUBMIT flag = 0 if no submit operation in process
subflg          byte    0               ;=255 if sumbit
;submit FCB
subfcb          byte    0               ;will be drive A
                byte    '$$$     '      ;name field
                byte    'SUB'           ;type field
                byte    0               ;extent
                byte    0,0             ;unused
SUBRC           byte    0               ;number of records
                defs    16              ;block number area
SUBCR           byte    0               ;current record to read
;
;com type field
typcom          byte    "COM"
;
;variable storage
cnbfpt          byte    0               ;buffer index
dfldsk          byte    0               ;default drive
tmpdsk          byte    0               ;temporary drive
dsksav          byte    0               ;save default during $$$.sub ops
count           byte    0               ;counter
curpnt          byte    0               ;input parse start
diradd          byte    0               ;directory number
savx            byte    0               ;save x location
savy            byte    0               ;save y location
length          byte    0               ;number pages to save
oldtmp          byte    0               ;temp drive in ren
dskbuf          word    tea             ;disk buffer start
across          byte    0               ;dir count on line
moduls          byte    9               ;modulus-1 for number input
num             word    0               ;input number
tnum            word    0               ;temp in bldnum
sadr            word    tea             ;save start address
DoLoc           word    tea             ;GO destination
;
;PEM section
                org     $/256*256+256   ;Align to Page address
;
pemram          jmp     pem
;pem variable storage
drvcmd          byte    0               ;new commanded drive
drvflg          byte    0               ;0 = nothing changed,
;                                       255 - drive number destroyed
usrcde          byte    0               ;user code
skpdir          byte    0               ;positive if no change
fcbind          byte    0               ;index to block number
savext          byte    0               ;save extent
frscol          byte    0               ;first col
pndkey          byte    0               ;pending input
lstflg          byte    0               ;printer flag
positn          byte    0               ;print position
; cmdinp          byte    0               ;input command - no longer used
addout          word    0               ;output address
bytout          equ     addout          ;output value
bufpsn          byte    0               ;input buffer position
exrwfl          byte    0               ;extend flag
tmpdrv          byte    0               ;temporary drive flag & number
countr          word    0               ;record counter
                byte    0               ;overflow
lkdown          word    0               ;down pnt. for block search
lookup          word    0               ;up pnt. for block search
olddrv          byte    0               ;old drive number
curdrv          byte    0               ;current drive
lginvc          byte    0               ;log in status
ronlst          byte    0               ;read write status
dirnum          word    0               ;directory number
subrec          byte    0               ;directory offset
recnum          word    0               ;record number
blknum          equ     recnum          ;block number
                byte    0               ;overflow
chrcnt          byte    0               ;character count
cmppnt          byte    0               ;comparison pointer
nxtrec          byte    0               ;next record
numrec          byte    0               ;number records
dirrec          word    0               ;directory record
dirmod          byte    0               ;directory mod 4
index           byte    0               ;buffer index
numcnt          byte    0               ;counter
outflg          byte    0               ;output enable flag
lstcol          byte    0               ;last column
mpdrsy          byte    0               ;save for y in mapdir
mpdrtm          byte    0               ;temp in mapdir
blmode          byte    0               ;<128 if byte else word
maxdrc          word    0               ;max directory record
sab             byte    0               ;mask for block
sxb             byte    0               ;shift for block
rtclk           byte    0,0,0           ;real time clock
gpcnt           word    0               ;gp counter
trkctr          byte    0               ;track counter
;
;following region is used to capture dcb
dcb
maxblk          word    0               ;maximum block number
sectrk          word    0               ;sectors per track
nsystr          word    0               ;number system tracks
blkscd          byte    0               ;block size code
maxdir          word    0               ;maximum directory number
alcmap          word    0               ;address of allocation map
chkflg          byte    0               ;check flag
chkmap          word    0               ;address of checksum map
wrtype          byte    0               ;write type 0=norm,1=dir,2=unalloc
exm             byte    0               ;extent mask
cexm1f          byte    0               ;exm complemented and 1f
;
;SIM section
; Note: the start of SIM RAM has been changed to a half-page address!
; Nothing "should" require any additional code changes.
; However (there's always one!), The XSBC123 Xmodem utility did
; require a change in the startup, as it was grabbing the jmp
; addresses for monitor routines, but assuming everything was
; on a page boundary (only grabbing the high order byte) - KM
;
                org     $/256*256+128   ;Align to Page address
;
simram          jmp     sim_boot        ;from cold start
sim_wboote      jmp     sim_wboot       ;from warm boot
                jmp     mon_consts      ;check for input
                jmp     mon_conrde      ;get input
                jmp     mon_conwrt      ;send to terminal
                nop                     ;printer output
                nop
                rts
                jmp     mon_punch       ;punch output
                jmp     mon_reader      ;reader input
                nop                     ;home drive
                nop
                rts
                jmp     sim_seldsk      ;select disk
                jmp     sim_seltrk      ;set track
                jmp     sim_selsec      ;set sector
                jmp     sim_setdma      ;set buffer address
                jmp     sim_read        ;read sector
                jmp     sim_write       ;write sector
                lda     #1              ;printer always ready
                rts
                ldx     #128            ;clock entry - disable for now
                rts
                jmp     sim_xlate       ;translate (goes to a RTS, but not used)
;
;console definition block
                byte    0               ;scratch
sysdef          byte    8               ;backspace
                byte    1               ;clear to end of line
                byte    $0C             ;forward space
                byte    0               ;normal video
                byte    '^'             ;invert video
                byte    24              ;lines per screen
                byte    80              ;char per line
                byte    $0C             ;form feed
                byte    $1E             ;home
                byte    2               ;clear to end of screen
;
;sim data area
sim_sekdsk      byte    0               ;seek disk number
sim_hstwrt      byte    0               ;0=written,1=pending host write
;allocate the following data areas to unused ram space
sim_hstact      defs    1               ;host active flag
sim_unacnt      defs    1               ;unalloc rec cnt
sim_sektrk      defs    2               ;seek track number
sim_seksec      defs    2               ;seek sector number
sim_hstdsk      defs    1               ;host disk number
sim_hsttrk      defs    2               ;host track number
sim_hstsec      defs    2               ;host sector number
sim_sekhst      defs    2               ;seek shr secshf
sim_unadsk      defs    1               ;last unalloc disk
sim_unatrk      defs    2               ;last unalloc track
sim_unasec      defs    2               ;last unalloc sector
sim_erflag      defs    1               ;error reporting
sim_rsflag      defs    1               ;read sector flag
sim_readop      defs    1               ;1 if read operation
sim_wrtype      defs    1               ;write operation type
sim_d65spt      defs    2               ;dos/65 records/track
;
        org     $/256*256+256           ;Align to Page address
;
;Major change here. To conserve ROM space, the allocation maps for the disk
; drives are no longer stored in ROM. Therefore, they don't need to be copied
; to RAM during a cold start. During startup, the map space shown below
; is cleared out by a routine in the SIM module. It uses bufmap and bufmapend
; as start and end addresses. - KM
;
;With other changes, enough RAM space was saved to add access and allocation
; maps for three additional drives, now from: A - G (total of 7)
; Note that the hstbuf has been moved to low RAM at $0600, which is the
; default LBA buffer for the C02BIOS. - KM
;
bufmap
;
;allocation and checksum maps
;drive a
almpa           equ     bufmap
;drive b
almpb           equ     almpa+512
;drive c
almpc           equ     almpb+512
;drive d
almpd           equ     almpc+512
;drive e
almpe           equ     almpd+512
;drive f
almpf           equ     almpe+512
;drive g
almpg           equ     almpf+512
;
bufmapend       equ     almpg+512
;
        code
;main program
        org     romstrt
;
        include         DOS65-CCM-V304.ASM      ;Include CCM Module
;
        org     $/256*256+256                   ;Align to Page address
;
        include         DOS65-PEM-V304.ASM      ;Include PEM Module
;
        org     $/256*256+256                   ;Align to Page address

        include         DOS65-SIM-V304.ASM      ;Include SIM Module
;
        ends
;
        data
        byte    $65                             ;Identifier
        ends
;
        end

